#ifndef _CLIB_H
#define _CLIB_H

#include <pthread.h>

#ifdef ANDROID

// Required for complex support in android
#ifndef I
#if __STDC_VERSION__ < 199901
#define _Complex __complex__
#endif
#define _Complex_I ((float _Complex)1.0i)

#define complex _Complex
#define I _Complex_I

#define creal(f) (__real__ (f))
#define cimag(f) (__imag__ (f))

#define conjf(f) (__real__ (f) - I * __imag__(f))
#define cabsf(f) sqrtf(__real__(f) * __real__(f) + __imag__(f) * __imag__(f))
#endif

// More android breakage
typedef struct pthread_barrier_t {
	volatile unsigned int count;
	unsigned int limit;
	pthread_mutex_t mutex;
	pthread_cond_t cond;
} pthread_barrier_t;

typedef struct pthread_barrrierattr_t pthread_barrierattr_t;

int pthread_barrier_destroy(pthread_barrier_t *barrier);
int pthread_barrier_init(pthread_barrier_t *barrier, const pthread_barrierattr_t *attr, unsigned count);
int pthread_barrier_wait(pthread_barrier_t *barrier);

#define HAVE_DECL_MEMALIGN 1

#include <android/log.h>
#define LOG(...) __android_log_print(ANDROID_LOG_INFO, "fx-engine", __VA_ARGS__)

#else // if android

#define HAVE_PTHREAD_BARRIER 1
#define HAVE_DECL_POSIX_MEMALIGN 1

#include <complex.h>

#define LOG(...) printf(__VA_ARGS__)

#endif

#include <stdlib.h>

void *xmemalign(size_t align, size_t size);

#endif
