/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _FX_IMAGE_H
#define _FX_IMAGE_H

#include "clib.h"

// complex multiply
void cmult(complex float *dst, complex float *a, complex float *b, int len);

// 4-channel byte to 2-channel complex float
void int8888_to_cf(uint8_t *src, int len, complex float *dsta, complex float *dstb);
// 2-channel complex float to 4-channel byte
void cf_to_int8888(complex float *srca, complex float *srcb, int len,uint8_t *dst, float scale);
// 3-channel short 565 to 2-channel complex float
void short565_to_cf(uint16_t *src, int len, complex float *dsta, complex float *dstb);
// 2-channel complex float to 3-channel short 565
void cf_to_short565(complex float *srca, complex float *srcb, int len,uint16_t *dst, float scale);

void transpose_cf_inplace(complex float *src, int w, int h);
void transpose_rgba_inplace(void *srcp, int w, int h);

static __inline__ int fx_shift(int fi, int fmax) {
	return fi < fmax/2 ? fi : fmax-fi;
}

static __inline__ float clampf(float v, float l, float u) {
	return v < l ? l : (v < u ? v : u);
}

static __inline__ int clampi(int v, int l, int u) {
	return v < l ? l : (v < u ? v : u);
}

static __inline__ int maxi(int a, int b) {
	return a > b ? a : b;
}

static __inline__ int mini(int a, int b) {
	return a < b ? a : b;
}

static __inline__ float maxf(float a, float b) {
	return a > b ? a : b;
}

static __inline__ float minf(float a, float b) {
	return a < b ? a : b;
}

#endif
