/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _FX_H
#define _FX_H

#define ENABLE_THREADS 1


#include <pthread.h>

#include "clib.h"

// assif
#define MAX_THREADS 8

typedef struct _ffts_plan_t ffts_plan_t;
typedef struct fxinfo fxinfo;

/**
 * Channel id for effect callbacks
 */
typedef enum fx_channel_t {
	FX_RG,
	FX_BA
} fx_channel_t;

typedef void (*fraw_to_cf_t)(void *src, int len, complex float *dsta, complex float *dstb);
typedef void (*fcf_to_raw_t)(complex float *srca, complex float *srcb, int len, void *dst, float scale);

typedef void (*fx_free_t)(void *data);


/**
 * Effect processor - processes a single row of frequency-domain data
 */
typedef void (*fx_effect_t)(fxinfo *fx, complex float *row, int len, int xi, int xlen, fx_channel_t cid, void *data);

enum FXCommand {
	FX_QUIT = 0,
	FX_CALL = 1,
};
typedef void (*fx_thread_call)(fxinfo *fx, int tid, void *data);

struct fxinfo {
	int fwidth, fheight;
	complex float *row0 __attribute__((aligned(64)));
	complex float *row1 __attribute__((aligned(64)));
	complex float *row2 __attribute__((aligned(64)));
	complex float *tmpa __attribute__((aligned(64)));
	complex float *tmpb __attribute__((aligned(64)));

	// Stored FFT of input signal for preview mode
	complex float *Ia __attribute__((aligned(64)));
	complex float *Ib __attribute__((aligned(64)));

	// Stored float version of input signal
	complex float *ia __attribute__((aligned(64)));
	complex float *ib __attribute__((aligned(64)));

	// Frequency modulator
	float *filter __attribute__((aligned(64)));
	uint8_t filter_enable;

	ffts_plan_t *fft_row;
	ffts_plan_t *ifft_row;
	ffts_plan_t *fft_col;
	ffts_plan_t *ifft_col;

#ifdef ENABLE_THREADS
	// For thread support
	int nthreads;
	pthread_t threads[MAX_THREADS];
	// Used to communicate synchronous command to worker threads & controller thread
	pthread_barrier_t cmd_barrier;
	// Used to communicate within worker threads
	pthread_barrier_t work_barrier;
	volatile int cmd;
	void *cmd_data;
	fx_thread_call cmd_call;	
#endif
	int efx_type;
	struct fxe_t *efx;
};

fxinfo *fx_create(int fwidth, int fheight, int nthreads);
void fx_free(fxinfo *fx);

void fx_call_threads(fxinfo *fx, fx_thread_call call, void *data);

#endif
