/*
 * Copyright (C) 2011 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nz.ac.waikato.picfx;

import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentTransaction;
import android.os.Bundle;

/**
 * Taken from FragmentTabs example
 *  * @author notzed
 */
public class FragmentTabListener<T extends Fragment> implements ActionBar.TabListener {

	private final Activity activity;
	private final String tag;
	private final Class<T> klass;
	private final Bundle args;
	private Fragment fragment;

	public FragmentTabListener(Activity activity, String tag, Class<T> clz) {
		this(activity, tag, clz, null);
	}

	public FragmentTabListener(Activity activity, String tag, Class<T> clz, Bundle args) {
		this.activity = activity;
		this.tag = tag;
		this.klass = clz;
		this.args = args;

		// Check to see if we already have a fragment for this tab, probably
		// from a previously saved state.  If so, deactivate it, because our
		// initial state is that a tab isn't shown.
		fragment = activity.getFragmentManager().findFragmentByTag(tag);
		if (fragment != null && !fragment.isDetached()) {
			FragmentTransaction ft = activity.getFragmentManager().beginTransaction();
			ft.detach(fragment);
			ft.commit();
		}
	}

	public void onTabSelected(Tab tab, FragmentTransaction ft) {
		if (fragment == null) {
			fragment = Fragment.instantiate(activity, klass.getName(), args);
			ft.add(android.R.id.content, fragment, tag);
		} else {
			ft.attach(fragment);
		}
	}

	public void onTabUnselected(Tab tab, FragmentTransaction ft) {
		if (fragment != null) {
			ft.detach(fragment);
		}
	}

	public void onTabReselected(Tab tab, FragmentTransaction ft) {
	}
}