/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package nz.ac.waikato.picfx.fx;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import nz.ac.waikato.picfx.R;
import nz.ac.waikato.picfx.util.StopWatch;

/**
 *
 * @author notzed
 */
public class Glare extends NativeEffect {

	float sigma = 7;
	float Ir = 1;
	float Ig = 1;
	float Ib = 1;
	float amount = 0.5f;

	@Override
	public String toString() {
		return String.format("[Glare sigma=%8.5f colour=%5.3f %5.3f %5.3f amount=%8.5f]",
				sigma, Ir, Ig, Ib, amount);
	}

	@Override
	protected void setProcess(EffectEngine ee) {
		System.out.println(this);
		ee.setProcess(EffectEngine.EFFECT_GLARE, null, sigma, Ir, Ig, Ib, amount);
	}

	@Override
	protected int estimateOverlap() {
		return (int) (sigma * 12.8);
	}

	@Override
	public View createView(Context c, final nz.ac.waikato.picfx.fx.Effect.EffectListener el) {
		LayoutInflater inflater = (LayoutInflater) c.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View view = inflater.inflate(R.layout.glare, null);

		final TextView sigmaLabel = (TextView) view.findViewById(R.id.sigma_label);
		SeekBar sigmaVal = (SeekBar) view.findViewById(R.id.sigma);
		final TextView amountLabel = (TextView) view.findViewById(R.id.amount_label);
		SeekBar amountVal = (SeekBar) view.findViewById(R.id.amount);

		sigmaVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				sigma = (val / 100.0f) + 0.5f;
				sigmaLabel.setText(String.format("Sigma: %5.3f", sigma));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		amountVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				amount = val / 1000.0f;
				amountLabel.setText(String.format("Amount: %5.3f", amount));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});

		amountLabel.setText(String.format("Amount: %5.3f", amount));
		sigmaLabel.setText(String.format("Sigma: %5.3f", sigma));

		sigmaVal.setProgress((int) ((sigma - 0.5f) * 100));
		amountVal.setProgress((int) (amount * 1000));

		return view;
	}
}
