/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package nz.ac.waikato.picfx.fx;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import nz.ac.waikato.picfx.R;
import nz.ac.waikato.picfx.util.StopWatch;

/**
 * Glazing effect.  Allows for a range of effects from emboss to glassy.
 * @author notzed
 */
public class Glaze extends NativeEffect {

	float sigma = 7;
	float ka = 0.8f;
	float kd = 0.2f;
	float ks = 0.3f;
	float shininess = 50;
	float Sr = 1, Sg = 1, Sb = 1;

	@Override
	protected void setProcess(EffectEngine ee) {
		System.out.println(this);
		ee.setProcess(EffectEngine.EFFECT_GLAZE, null,
				sigma, ka, kd, ks, shininess,
				1, 1, 1,
				1, 1, 1,
				0, 0, -1,
				0, 0, -1);
	}

	@Override
	protected int estimateOverlap() {
		return (int) (sigma * 12.8);
	}

	@Override
	public View createView(Context c, final nz.ac.waikato.picfx.fx.Effect.EffectListener el) {
		LayoutInflater inflater = (LayoutInflater) c.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View view = inflater.inflate(R.layout.glaze, null);

		final TextView sigmaLabel = (TextView) view.findViewById(R.id.sigma_label);
		SeekBar sigmaVal = (SeekBar) view.findViewById(R.id.sigma);
		final TextView ambientLabel = (TextView) view.findViewById(R.id.ambient_label);
		SeekBar ambientVal = (SeekBar) view.findViewById(R.id.ambient);
		final TextView diffuseLabel = (TextView) view.findViewById(R.id.diffuse_label);
		SeekBar diffuseVal = (SeekBar) view.findViewById(R.id.diffuse);
		final TextView specularLabel = (TextView) view.findViewById(R.id.specular_label);
		SeekBar specularVal = (SeekBar) view.findViewById(R.id.specular);
		//final TextView shueLabel = (TextView) view.findViewById(R.id.shue_label);
		//SeekBar shueVal = (SeekBar) view.findViewById(R.id.shue);
		final TextView shininessLabel = (TextView) view.findViewById(R.id.shininess_label);
		SeekBar shininessVal = (SeekBar) view.findViewById(R.id.shininess);

		sigmaVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				sigma = (val / 100.0f) + 0.5f;
				sigmaLabel.setText(String.format("Sigma: %5.3f", sigma));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		ambientVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				ka = val / 1000.0f;
				ambientLabel.setText(String.format("Ambient: %5.3f", ka));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		diffuseVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				kd = val / 1000.0f;
				diffuseLabel.setText(String.format("Diffuse: %5.3f", kd));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		specularVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				ks = val / 1000.0f;
				specularLabel.setText(String.format("Specular: %5.3f", ks));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		/*
		 shueVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
		 public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
		 ks = val / 1000.0f;
		 if (arg2)
		 el.effectUpdated();
		 }

		 public void onStartTrackingTouch(SeekBar arg0) {
		 }

		 public void onStopTrackingTouch(SeekBar arg0) {
		 }
		 });*/
		shininessVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				shininess = val + 1;
				shininessLabel.setText(String.format("Shininess: %5.3f", shininess));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});

		ambientLabel.setText(String.format("Ambient: %5.3f", ka));
		sigmaLabel.setText(String.format("Sigma: %5.3f", sigma));
		diffuseLabel.setText(String.format("Diffuse: %5.3f", kd));
		specularLabel.setText(String.format("Specular: %5.3f", ks));
		shininessLabel.setText(String.format("Shininess: %5.3f", shininess));

		sigmaVal.setProgress((int) ((sigma - 0.5f) * 100));
		ambientVal.setProgress((int) (ka * 1000));
		diffuseVal.setProgress((int) (kd * 1000));
		specularVal.setProgress((int) (ks * 1000));
		shininessVal.setProgress((int) ((shininess - 1)));

		return view;
	}
}
