/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package nz.ac.waikato.picfx.fx;

import android.graphics.Bitmap;
import nz.ac.waikato.picfx.util.StopWatch;

/**
 * Class for all native effects (implemented in jni layer)
 * @author notzed
 */
public abstract class NativeEffect extends Effect {

	protected abstract void setProcess(EffectEngine ee);
	protected abstract int estimateOverlap();

	@Override
	public void loadPreview(EffectEngine ee, Bitmap input, int x, int y) {
		setProcess(ee);
		ee.loadPreview(input, x, y);
	}

	@Override
	public void apply(Bitmap input, Bitmap output) {
		StopWatch t = new StopWatch();

		t.start();

		EffectEngine ee = EffectEngine.get();

		setProcess(ee);

		t.mark("update kernel");

		// FIXME: calculate overlap properly in C end
		ee.process(input, output, estimateOverlap());

		t.mark("process image");

		t.finish();
		t.print(System.out);
	}

	@Override
	public void updatePreview(EffectEngine ee, Bitmap output) {
		StopWatch t = new StopWatch();

		t.start();

		setProcess(ee);

		t.mark("update kernel");

		ee.processPreview(output);

		t.mark("process preview");

		t.finish();
		t.print(System.out);
	}
}
