/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package nz.ac.waikato.picfx.fx;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import nz.ac.waikato.picfx.R;

/**
 * Gaussian blur
 * @author notzed
 */
public class SoftFocus extends NativeEffect {

	float sigma = 7;
	float blend = 0.5f;

	public SoftFocus() {
	}

	@Override
	protected void setProcess(EffectEngine ee) {
		ee.setProcess(EffectEngine.EFFECT_BLUR, null, sigma, blend);
	}

	@Override
	protected int estimateOverlap() {
		return (int) (sigma * 12.8);
	}

	@Override
	public View createView(Context c, final EffectListener el) {
		LayoutInflater inflater = (LayoutInflater) c.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View view = inflater.inflate(R.layout.soft_focus, null);

		final TextView sigmaLabel = (TextView) view.findViewById(R.id.sigma_label);
		SeekBar sigmaVal = (SeekBar) view.findViewById(R.id.sigma);
		final TextView blendLabel = (TextView) view.findViewById(R.id.blend_label);
		SeekBar blendVal = (SeekBar) view.findViewById(R.id.blend);

		sigmaVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				sigma = val / 100.0f + 0.5f;
				sigmaLabel.setText(String.format("Sigma: %5.3f", sigma));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		sigmaVal.setProgress((int) ((sigma - 0.5f)* 100));
		blendVal.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			public void onProgressChanged(SeekBar arg0, int val, boolean arg2) {
				blend = val / 1000.0f;
				blendLabel.setText(String.format("Blend: %5.3f", blend));
				if (arg2)
					el.effectUpdated();
			}

			public void onStartTrackingTouch(SeekBar arg0) {
			}

			public void onStopTrackingTouch(SeekBar arg0) {
			}
		});
		blendVal.setProgress((int) (blend * 1000));

		return view;
	}
}
