/*
 * Copyright 2013, The University of Waikato.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 	* Neither the name of the organization nor the
 *       names of its contributors may be used to endorse or promote products
 * 	  derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package nz.ac.waikato.picfx.util;

import java.io.PrintStream;

/**
 * Timing tool
 * @author notzed
 */
public class StopWatch {

	long[] stamps = new long[32];
	String[] whats = new String[32];
	int index;

	public void start() {
		stamps[0] = System.nanoTime();
		index = 1;
	}

	public void mark(String what, Object... args) {
		stamps[index] = System.nanoTime();
		whats[index] = String.format(what, args);
		index += 1;
	}

	public void finish() {
		stamps[index] = System.nanoTime();
	}

	public void print(PrintStream o) {
		for (int i = 1; i < index; i++) {
			long diff = stamps[i] - stamps[i - 1];

			System.out.printf("%s took %d.%09ds\n",
					whats[i],
					diff / 1000000000L, diff % 1000000000L);
		}
	}
}
